@@ -1,9 +1,9 @@
+from dataclasses import dataclass
 from datetime import datetime
 
-from typing import TypedDict
 
-
-class EmployeeDTO(TypedDict):
+@dataclass
+class EmployeeDTO():
     restaurant: str
     name: str
     altname: str
@@ -13,7 +13,8 @@ class EmployeeDTO(TypedDict):
     ident: int
 
 
-class OrderListDTO(TypedDict):
+@dataclass
+class OrderListDTO():
     visit_id: int
     count_guest: int
     order_id: int
@@ -21,7 +22,7 @@ class OrderListDTO(TypedDict):
     url: str
     version: int
     crc32: int
-    guid: set
+    guid: str
     table_id: int
     table_code: int
     order_categ_id: int
@@ -41,27 +42,31 @@ class OrderListDTO(TypedDict):
     finished_time: datetime
 
 
-class WaiterListDTO(TypedDict):
+@dataclass
+class WaiterListDTO():
     id: int
     code: int
 
 
-class RestaurantsDTO(TypedDict):
+@dataclass
+class RestaurantsDTO():
     name: str
     alt_name: str
     ident: int
 
 
-class MenuDTO(TypedDict):
+@dataclass
+class MenuDTO():
     ident: int
     price: int
 
 
-class MenuItemsDTO(TypedDict):
+@dataclass
+class MenuItemsDTO():
     ident: int
     item_ident: int
     source_ident: int
-    guid_string: set
+    guid: str
     assign_childs_on_server: bool
     active_hierarchy: bool
     code: int
@@ -94,19 +99,12 @@ class MenuItemsDTO(TypedDict):
     use_confirm_qnt: str
     bar_codes_full_info: str
     item_kind: str
-    high_level_group4: str
-    high_level_group3: str
-    high_level_group2: str
-    high_level_group1: str
-    large_image_path: str
     combo_discount: str
     tariff_round_rule: str
-    trr_math: str
     money_round_rule: str
     round_time: str
     guests_dish_rating: str
     rate_type: str
-    rt_one_time: str
     minimum_tarif_time: str
     maximum_tarif_time: str
     ignored_tarif_time: str
@@ -119,23 +117,25 @@ class MenuItemsDTO(TypedDict):
     portion_name: str
 
 
-class WaiterDTO(TypedDict):
+@dataclass
+class WaiterDTO():
     restaurant: str
     name: str
     ident: int
     code: int
 
 
-class GetOrderDTO(TypedDict):
-    ident: int
-    guid: dict
+@dataclass
+class OrderLineDTO():
+    order_guid: str
     dish_id: int
     dish_name: str
     quantity: int
     modi: str
 
 
-class GetVisitDTO(TypedDict):
+@dataclass
+class GetVisitDTO():
     order_id: int
     guid: str
 
